@@ -12,6 +12,9 @@
 #include <locale.h>
 #include <argp.h>
 #include <elfutils/libdwfl.h>
+#ifdef HAVE_ELFUTILS_VERSION_H
+#include <elfutils/version.h>
+#endif
 #include <dwarf.h>
 #include <obstack.h>
 #include <unistd.h>
@@ -75,7 +78,7 @@ get_location (Dwarf_Addr dwbias, Dwarf_Addr pc, Dwarf_Attribute *loc_attr,
 static void
 handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 		 Dwarf_Addr cubias, Dwarf_Die *vardie, Dwarf_Addr pc,
-		 char **fields)
+		 Dwarf_Op *cfa_ops, char **fields)
 {
 #define obstack_chunk_alloc malloc
 #define obstack_chunk_free free
@@ -87,7 +90,8 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
      XXX inlines botched
   */
   Dwarf_Attribute fb_attr_mem, *fb_attr = NULL;
-  for (int inner = 0; inner < nscopes; ++inner)
+  int inner;
+  for (inner = 0; inner < nscopes; ++inner)
     {
       switch (dwarf_tag (&scopes[inner]))
 	{
@@ -120,7 +124,7 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
   struct location *head, *tail = NULL;
   head = c_translate_location (&pool, &fail, NULL, NULL,
 			       1, cubias, pc, locexpr, locexpr_len,
-			       &tail, fb_attr);
+			       &tail, fb_attr, cfa_ops);
 
   if (dwarf_attr_integrate (vardie, DW_AT_type, &attr_mem) == NULL)
     error (2, 0, _("cannot get type of variable: %s"),
@@ -213,10 +217,35 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
 	    }
 	  else
 	    {
-	      locexpr = get_location (cubias, pc, &attr_mem, &locexpr_len);
-	      c_translate_location (&pool, NULL, NULL, NULL,
-				    1, cubias, pc, locexpr, locexpr_len,
-				    &tail, NULL);
+	      /* We are expection a block, constant or loclistptr. */
+	      unsigned int form = dwarf_whatform (&attr_mem);
+	      Dwarf_Sword off;
+	      switch (form)
+		{
+		/* constant */
+		case DW_FORM_data1:
+		case DW_FORM_data2:
+		case DW_FORM_sdata:
+		case DW_FORM_udata:
+		  if (dwarf_formsdata (&attr_mem, &off) != 0)
+		    error (2, 0, _("Bad offset for %s %s: %s"),
+			   typetag == DW_TAG_union_type ? "union" : "struct",
+			   dwarf_diename_integrate (die) ?: "<anonymous>",
+			   dwarf_errmsg (-1));
+		    if (off != 0)
+		      c_translate_add_offset (&pool, 1,
+					      dwarf_diename_integrate (die)
+					      ?: "", off, &tail);
+		  break;
+
+		default:
+		    locexpr = get_location (cubias, pc, &attr_mem,
+					    &locexpr_len);
+		    c_translate_location (&pool, NULL, NULL, NULL,
+					  1, cubias, pc, locexpr, locexpr_len,
+					  &tail, NULL, NULL);
+		    break;
+		}
 	    }
 	  ++fields;
 	  break;
@@ -254,7 +283,9 @@ handle_variable (Dwarf_Die *scopes, int nscopes, int out,
       if (typedie == NULL)
 	error (2, 0, _("cannot get type of field: %s"), dwarf_errmsg (-1));
       typetag = dwarf_tag (typedie);
-      if (typetag != DW_TAG_typedef)
+      if (typetag != DW_TAG_typedef &&
+	  typetag != DW_TAG_const_type &&
+	  typetag != DW_TAG_volatile_type)
 	break;
       if (dwarf_attr_integrate (typedie, DW_AT_type, &attr_mem) == NULL)
 	error (2, 0, _("cannot get type of field: %s"), dwarf_errmsg (-1));
@@ -375,6 +406,9 @@ static void
 print_vars (unsigned int indent, Dwarf_Die *die)
 {
   Dwarf_Die child;
+  Dwarf_Attribute attr_mem;
+  Dwarf_Die typedie_mem;
+  Dwarf_Die *typedie;
   if (dwarf_child (die, &child) == 0)
     do
       switch (dwarf_tag (&child))
@@ -384,9 +418,7 @@ print_vars (unsigned int indent, Dwarf_Die *die)
 	  printf ("%*s%-30s[%6" PRIx64 "]", indent, "",
 		  dwarf_diename (&child),
 		  (uint64_t) dwarf_dieoffset (&child));
-	  Dwarf_Attribute attr_mem;
-	  Dwarf_Die typedie_mem;
-	  Dwarf_Die *typedie = dwarf_formref_die
+	  typedie = dwarf_formref_die
 	    (dwarf_attr_integrate (&child, DW_AT_type, &attr_mem),
 	     &typedie_mem);
 	  print_type (typedie, '\t');
@@ -492,7 +524,41 @@ main (int argc, char **argv)
 	error (0, 0, "dwarf_getscopevar: %s (+%d, %s:%d:%d): %s",
 	       spec, shadow, at, lineno, colno, dwarf_errmsg (-1));
       else
-	handle_variable (scopes, n, out, cubias, &vardie, pc, &argv[argi]);
+	{
+	  Dwarf_Op *cfa_ops = NULL;
+
+#ifdef _ELFUTILS_PREREQ
+#if _ELFUTILS_PREREQ(0,142)
+	  size_t cfa_nops;
+	  Dwarf_Addr bias;
+	  Dwfl_Module *module = dwfl_addrmodule (dwfl, pc);
+	  if (module != NULL)
+	    {
+	      // Try debug_frame first, then fall back on eh_frame.
+	      Dwarf_CFI *cfi = dwfl_module_dwarf_cfi (module, &bias);
+	      if (cfi != NULL)
+		{
+		  Dwarf_Frame *frame = NULL;
+		  if (dwarf_cfi_addrframe (cfi, pc, &frame) == 0)
+		    dwarf_frame_cfa (frame, &cfa_ops, &cfa_nops);
+		}
+	      if (cfa_ops == NULL)
+		{
+		  cfi = dwfl_module_eh_cfi (module, &bias);
+		  if (cfi != NULL)
+		    {
+		      Dwarf_Frame *frame = NULL;
+		      if (dwarf_cfi_addrframe (cfi, pc, &frame) == 0)
+			dwarf_frame_cfa (frame, &cfa_ops, &cfa_nops);
+		    }
+		}
+	    }
+#endif
+#endif	  
+
+	  handle_variable (scopes, n, out, cubias, &vardie, pc, cfa_ops,
+			   &argv[argi]);
+	}
     }
 
   free (scopes);
@@ -501,3 +567,5 @@ main (int argc, char **argv)
 
   return 0;
 }
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
