@@ -48,7 +48,11 @@ struct location
       unsigned int stack_depth;	/* Temporaries "s0..<N>" used by it.  */
       bool used_deref;		/* Program uses "deref" macro.  */
     } address;
-    unsigned int regno;		/* loc_register */
+    struct			/* loc_register */
+    {
+      unsigned int regno;
+      Dwarf_Word offset;
+    } reg;
     struct location *pieces;	/* loc_noncontiguous */
   };
 };
@@ -182,7 +186,7 @@ translate (struct obstack *pool, int indent, Dwarf_Addr addrbias,
 	break;
 
       case loc_register:
-	tos_register = input->regno;
+	tos_register = input->reg.regno;
 	break;
 
       default:
@@ -215,7 +219,8 @@ translate (struct obstack *pool, int indent, Dwarf_Addr addrbias,
       else
 	{
 	  piece->type = loc_register;
-	  piece->regno = tos_register;
+	  piece->reg.regno = tos_register;
+	  piece->reg.offset = 0;
 	}
       return NULL;
     }
@@ -246,9 +251,11 @@ translate (struct obstack *pool, int indent, Dwarf_Addr addrbias,
 	  break;
 
 	case DW_OP_drop:
-	  POP (ignore);
-	  emit ("%*s/* drop " STACKFMT "*/\n", indent * 2, "", ignore);
-	  break;
+	  {
+	    POP (ignore);
+	    emit ("%*s/* drop " STACKFMT "*/\n", indent * 2, "", ignore);
+	    break;
+	  }
 
 	case DW_OP_pick:
 	  sp = expr[i].number;
@@ -502,6 +509,15 @@ translate (struct obstack *pool, int indent, Dwarf_Addr addrbias,
 	    }
 	  break;
 
+	case DW_OP_call_frame_cfa:
+	  // We pick this out when processing DW_AT_frame_base in
+	  // so it really shouldn't turn up here.
+	  if (need_fb == NULL)
+	    DIE ("DW_OP_call_frame_cfa while processing frame base");
+	  else
+	    DIE ("DW_OP_call_frame_cfa not expected outside DW_AT_frame_base");
+	  break;
+
 	case DW_OP_push_object_address:
 	  DIE ("XXX DW_OP_push_object_address");
 	  break;
@@ -545,7 +561,8 @@ location_from_address (struct obstack *pool,
 					     struct obstack *, Dwarf_Addr),
 		       int indent, Dwarf_Addr dwbias,
 		       const Dwarf_Op *expr, size_t len, Dwarf_Addr address,
-		       struct location **input, Dwarf_Attribute *fb_attr)
+		       struct location **input, Dwarf_Attribute *fb_attr,
+		       const Dwarf_Op *cfa_ops)
 {
   struct location *loc = obstack_alloc (pool, sizeof *loc);
   loc->fail = *input == NULL ? fail : (*input)->fail;
@@ -592,8 +609,19 @@ location_from_address (struct obstack *pool,
 	  return NULL;
 	}
 
+      // If it is DW_OP_call_frame_cfa then get cfi cfa ops.
+      const Dwarf_Op * fb_ops;
+      if (fb_len == 1 && fb_expr[0].atom == DW_OP_call_frame_cfa)
+	{
+	  if (cfa_ops == NULL)
+	    FAIL (loc, N_("No cfa_ops supplied, but needed by DW_OP_call_frame_cfa"));
+	  fb_ops = cfa_ops;
+	}
+      else
+	fb_ops = fb_expr;
+
       loc->frame_base = alloc_location (pool, loc);
-      failure = translate (pool, indent + 1, dwbias, fb_expr, fb_len, NULL,
+      failure = translate (pool, indent + 1, dwbias, fb_ops, fb_len, NULL,
 			   NULL, &loser, loc->frame_base);
       if (failure != NULL)
 	return lose (loc, failure, fb_expr, loser);
@@ -615,9 +643,10 @@ static struct location *
 location_relative (struct obstack *pool,
 		   int indent, Dwarf_Addr dwbias,
 		   const Dwarf_Op *expr, size_t len, Dwarf_Addr address,
-		   struct location **input, Dwarf_Attribute *fb_attr)
+		   struct location **input, Dwarf_Attribute *fb_attr,
+		   const Dwarf_Op *cfa_ops)
 {
-  Dwarf_Sword *stack;
+  Dwarf_Sword *stack = NULL;
   unsigned int stack_depth = 0, max_stack = 0;
   inline void deepen (void)
     {
@@ -745,7 +774,8 @@ location_relative (struct obstack *pool,
 	  /* This started from a register, but now it's following a pointer.
 	     So we can do the translation starting from address here.  */
 	  return location_from_address (pool, NULL, NULL, NULL, indent, dwbias,
-					expr, len, address, input, fb_attr);
+					expr, len, address, input, fb_attr,
+					cfa_ops);
 
 
 	  /* Constant-value operations.  */
@@ -904,7 +934,8 @@ location_relative (struct obstack *pool,
 		    loc = location_from_address (pool, NULL, NULL, NULL,
 						 indent, dwbias,
 						 &expr[i + 1], len - i - 1,
-						 address, input, fb_attr);
+						 address, input, fb_attr,
+						 cfa_ops);
 		    if (loc == NULL)
 		      return NULL;
 		  }
@@ -914,7 +945,9 @@ location_relative (struct obstack *pool,
 	      }
 
 	    case loc_register:
-	      // XXX
+	      /* This piece (or the whole struct) fits in a register.  */
+	      (*input)->reg.offset += value;
+	      return head ?: *input;
 
 	    default:
 	      abort ();
@@ -985,7 +1018,8 @@ c_translate_location (struct obstack *pool,
 					    struct obstack *, Dwarf_Addr),
 		      int indent, Dwarf_Addr dwbias, Dwarf_Addr pc_address,
 		      const Dwarf_Op *expr, size_t len,
-		      struct location **input, Dwarf_Attribute *fb_attr)
+		      struct location **input, Dwarf_Attribute *fb_attr,
+		      const Dwarf_Op *cfa_ops)
 {
   indent += 2;
 
@@ -997,14 +1031,14 @@ c_translate_location (struct obstack *pool,
       return location_from_address (pool, fail, fail_arg,
 				    emit_address ?: &default_emit_address,
 				    indent, dwbias, expr, len, pc_address,
-				    input, fb_attr);
+				    input, fb_attr, cfa_ops);
 
     case loc_noncontiguous:
     case loc_register:
       /* The starting point is not an address computation, but a
 	 register.  We can only handle limited computations from here.  */
       return location_relative (pool, indent, dwbias, expr, len, pc_address,
-				input, fb_attr);
+				input, fb_attr, cfa_ops);
 
     default:
       abort ();
@@ -1014,17 +1048,53 @@ c_translate_location (struct obstack *pool,
   return NULL;
 }
 
+/* Translate a C fragment for a direct argument VALUE.  On errors, call FAIL,
+   which should not return.  Any later errors will use FAIL and FAIL_ARG from
+   this translate call.  On success, return the fragment created. */
+struct location *
+c_translate_argument (struct obstack *pool,
+                      void (*fail) (void *arg, const char *fmt, ...)
+                      __attribute__ ((noreturn, format (printf, 2, 3))),
+                      void *fail_arg,
+                      void (*emit_address) (void *fail_arg,
+                                            struct obstack *, Dwarf_Addr),
+                      int indent, const char *value)
+{
+  indent += 2;
+
+  obstack_printf(pool, "%*saddr = %s;\n", indent * 2, "", value);
+  obstack_1grow (pool, '\0');
+  char *program = obstack_finish (pool);
+
+  struct location *loc = obstack_alloc (pool, sizeof *loc);
+  loc->next = NULL;
+  loc->fail = fail;
+  loc->fail_arg = fail_arg;
+  loc->emit_address = emit_address;
+  loc->ops = NULL;
+  loc->nops = 0;
+  loc->byte_size = 0;
+  loc->type = loc_address;
+  loc->frame_base = NULL;
+  loc->address.declare = NULL;
+  loc->address.program = program;
+  loc->address.stack_depth = 0;
+  loc->address.used_deref = false;
+
+  return loc;
+}
+
+
 /* Emit "uintNN_t TARGET = ...;".  */
 static bool
 emit_base_fetch (struct obstack *pool, Dwarf_Word byte_size,
                  bool signed_p, const char *target, struct location *loc)
 {
   bool deref = false;
-  /* int i; */
 
-  /* Emit size/signed coercion. */ 
+  /* Emit size/signed coercion. */
   obstack_printf (pool, "{ ");
-  obstack_printf (pool, "%sint%u_t value = ", 
+  obstack_printf (pool, "%sint%u_t value = ",
                   (signed_p ? "" : "u"), (unsigned)(byte_size * 8));
 
   switch (loc->type)
@@ -1038,7 +1108,9 @@ emit_base_fetch (struct obstack *pool, Dwarf_Word byte_size,
       break;
 
     case loc_register:
-      obstack_printf (pool, "fetch_register (%u);", loc->regno);
+      if (loc->reg.offset != 0)
+	FAIL (loc, N_("cannot handle offset into register in fetch"));
+      obstack_printf (pool, "fetch_register (%u);", loc->reg.regno);
       break;
 
     case loc_noncontiguous:
@@ -1072,7 +1144,9 @@ emit_base_store (struct obstack *pool, Dwarf_Word byte_size,
       return true;
 
     case loc_register:
-      obstack_printf (pool, "store_register (%u, %s);", loc->regno, rvalue);
+      if (loc->reg.offset != 0)
+	FAIL (loc, N_("cannot handle offset into register in store"));
+      obstack_printf (pool, "store_register (%u, %s);", loc->reg.regno, rvalue);
       break;
 
     case loc_noncontiguous:
@@ -1371,7 +1445,7 @@ c_translate_fetch (struct obstack *pool, int indent,
   if (dwarf_attr_integrate (die, DW_AT_encoding, &encoding_attr) == NULL
       || dwarf_formudata (&encoding_attr, &encoding) != 0)
     encoding = base_encoding (typedie, *input);
-  bool signed_p = (encoding == DW_ATE_signed 
+  bool signed_p = (encoding == DW_ATE_signed
                    || encoding == DW_ATE_signed_char);
 
   *input = discontiguify (pool, indent, *input, byte_size,
@@ -1490,7 +1564,7 @@ c_translate_store (struct obstack *pool, int indent,
   if (dwarf_attr_integrate (die, DW_AT_encoding, &encoding_attr) == NULL
       || dwarf_formudata (&encoding_attr, &encoding) != 0)
     encoding = base_encoding (typedie, *input);
-  bool signed_p = (encoding == DW_ATE_signed 
+  bool signed_p = (encoding == DW_ATE_signed
                    || encoding == DW_ATE_signed_char);
 
   *input = discontiguify (pool, indent, *input, byte_size,
@@ -1591,6 +1665,68 @@ c_translate_addressof (struct obstack *pool, int indent,
 }
 
 
+/* Translate a fragment to write the given pointer value,
+   where *INPUT is the location of the pointer with that type.
+*/
+
+void
+c_translate_pointer_store (struct obstack *pool, int indent,
+                           Dwarf_Addr dwbias __attribute__ ((unused)),
+                           Dwarf_Die *typedie, struct location **input,
+                           const char *rvalue)
+{
+  assert (dwarf_tag (typedie) == DW_TAG_pointer_type);
+
+  Dwarf_Attribute attr_mem;
+  Dwarf_Word byte_size;
+  if (dwarf_attr_integrate (typedie, DW_AT_byte_size, &attr_mem) == NULL)
+    byte_size = 0;
+  else if (dwarf_formudata (&attr_mem, &byte_size) != 0)
+    FAIL (*input,
+	  N_("cannot get byte_size attribute for type %s: %s"),
+	  dwarf_diename (typedie) ?: "<anonymous>",
+	  dwarf_errmsg (-1));
+
+  translate_base_store (pool, indent + 1, byte_size,
+                        input, *input, rvalue);
+
+  // XXX: what about multiple-location lvalues?
+}
+
+
+/* Translate a fragment to add an offset to the currently calculated
+   address of the input location. Used for struct fields. Only works
+   when location is already an actual base address.
+*/
+
+void
+c_translate_add_offset (struct obstack *pool, int indent, const char *comment,
+                        Dwarf_Sword off, struct location **input)
+{
+  indent++;
+  if (comment == NULL || comment[0] == '\0')
+    comment = "field offset";
+  switch ((*input)->type)
+    {
+    case loc_address:
+      obstack_printf (pool, "%*saddr += " SFORMAT "; // %s\n",
+		      indent * 2 + 2, "", off, comment);
+      *input = (*input)->next = new_synthetic_loc (pool, *input, false);
+    break;
+
+    case loc_register:
+      FAIL (*input, N_("cannot add offset of object in register"));
+      break;
+    case loc_noncontiguous:
+      FAIL (*input, N_("cannot add offset of noncontiguous object"));
+      break;
+
+    default:
+      abort ();
+      break;
+    }
+}
+
 /* Determine the element stride of an array type.  */
 static Dwarf_Word
 array_stride (Dwarf_Die *typedie, struct location *origin)
@@ -1613,6 +1749,19 @@ array_stride (Dwarf_Die *typedie, struct location *origin)
 	  dwarf_diename (typedie) ?: "<anonymous>",
 	  dwarf_errmsg (-1));
 
+  int typetag = dwarf_tag(&die_mem);
+  while (typetag == DW_TAG_typedef ||
+         typetag == DW_TAG_const_type ||
+         typetag == DW_TAG_volatile_type)
+    {
+      if (dwarf_attr_integrate (&die_mem, DW_AT_type, &attr_mem) == NULL
+          || dwarf_formref_die (&attr_mem, &die_mem) == NULL)
+        FAIL (origin, N_("cannot get inner type of type %s: %s"),
+              dwarf_diename (&die_mem) ?: "<anonymous>",
+              dwarf_errmsg (-1));
+      typetag = dwarf_tag(&die_mem);
+    }
+
   if (dwarf_attr_integrate (&die_mem, DW_AT_byte_size, &attr_mem) != NULL)
     {
       Dwarf_Word stride;
@@ -1634,7 +1783,8 @@ c_translate_array (struct obstack *pool, int indent,
 		   Dwarf_Die *typedie, struct location **input,
 		   const char *idx, Dwarf_Word const_idx)
 {
-  assert (dwarf_tag (typedie) == DW_TAG_array_type);
+  assert (dwarf_tag (typedie) == DW_TAG_array_type ||
+          dwarf_tag (typedie) == DW_TAG_pointer_type);
 
   ++indent;
 
@@ -1677,7 +1827,15 @@ c_translate_array (struct obstack *pool, int indent,
       break;
 
     case loc_register:
-      FAIL (*input, N_("cannot index array stored in a register"));
+      if (idx != NULL)
+	FAIL (*input, N_("cannot index array stored in a register"));
+      else if (const_idx > max_fetch_size (loc, typedie) / stride)
+	FAIL (*input, N_("constant index is outside array held in register"));
+      else
+	{
+	  loc->reg.offset += const_idx * stride;
+	  return;
+	}
       break;
 
     default:
@@ -1728,8 +1886,11 @@ emit_loc_register (FILE *out, struct location *loc, unsigned int indent,
 {
   assert (loc->type == loc_register);
 
+  if (loc->reg.offset != 0)
+    FAIL (loc, N_("cannot handle offset into register in fetch"));
+
   emit ("%*s%s = fetch_register (%u);\n",
-	indent * 2, "", target, loc->regno);
+	indent * 2, "", target, loc->reg.regno);
 }
 
 /* Emit a code fragment to assign the target variable to an address.  */
@@ -1862,3 +2023,5 @@ c_emit_location (FILE *out, struct location *loc, int indent)
 }
 
 #undef emit
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
