@@ -20,6 +20,8 @@
 #include "util.h"
 #include "coveragedb.h"
 #include "git_version.h"
+#include "rpm_finder.h"
+#include "sys/sdt.h"
 
 #include <iostream>
 #include <fstream>
@@ -192,16 +194,17 @@ printscript(systemtap_session& s, ostream& o)
             {
               o << pp;
               // Print the locals for -L mode only
-              if (s.unoptimized) {
-                for (unsigned j=0; j<p->locals.size(); j++)
-                  {
-                    o << " ";
-                    vardecl* v = p->locals[j];
-                    v->printsig (o);
-                  }
-                // Print arguments of probe if there
-                p->printargs(o);
-		}
+              if (s.listing_mode_vars)
+                {
+                  for (unsigned j=0; j<p->locals.size(); j++)
+                    {
+                      o << " ";
+                      vardecl* v = p->locals[j];
+                      v->printsig (o);
+                    }
+                  // Print arguments of probe if there
+                  p->printargs(o);
+                }
               o << endl;
               seen.insert (pp);
             }
@@ -324,6 +327,34 @@ setup_signals (sighandler_t handler)
   sigaction (SIGTERM, &sa, NULL);
 }
 
+void
+setup_kernel_release (systemtap_session &s, const char* kstr) {
+    if (kstr[0] == '/') // fully specified path
+      {
+        s.kernel_build_tree = kstr;
+        string version_file_name = s.kernel_build_tree + "/include/config/kernel.release";
+        // The file include/config/kernel.release within the
+        // build tree is used to pull out the version information
+        ifstream version_file (version_file_name.c_str());
+        if (version_file.fail ())
+          {
+            cerr << "Missing " << version_file_name << endl;
+            exit(1);
+          }
+        else
+          {
+            char c;
+            s.kernel_release = "";
+            while (version_file.get(c) && c != '\n')
+              s.kernel_release.push_back(c);
+          }
+      }
+    else
+      {
+        s.kernel_release = string (kstr);
+        s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
+      }
+}
 
 int
 main (int argc, char * const argv [])
@@ -348,6 +379,7 @@ main (int argc, char * const argv [])
   s.unoptimized = false;
   s.suppress_warnings = false;
   s.listing_mode = false;
+  s.listing_mode_vars = false;
 
 #ifdef ENABLE_PROLOGUES
   s.prologue_searching = true;
@@ -375,6 +407,14 @@ main (int argc, char * const argv [])
   s.load_only = false;
   s.skip_badvars = false;
 
+  // Location of our signing certificate.
+  // If we're root, use the database in SYSCONFDIR, otherwise
+  // use the one in our $HOME directory.  */
+  if (geteuid() == 0)
+    s.cert_db_path = SYSCONFDIR "/systemtap/ssl/server";
+  else
+    s.cert_db_path = getenv("HOME") + string ("/.systemtap/ssl/server");
+
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
   if (s_p != NULL)
   {
@@ -426,37 +466,14 @@ main (int argc, char * const argv [])
 
   const char* s_kr = getenv ("SYSTEMTAP_RELEASE");
   if (s_kr != NULL) {
-    if (s_kr[0] == '/') // fully specified path
-      {
-        s.kernel_build_tree = s_kr;
-        string version_file_name = s.kernel_build_tree + "/include/config/kernel.release";
-        // The file include/config/kernel.release within the
-        // build tree is used to pull out the version information
-        ifstream version_file (version_file_name.c_str());
-        if (version_file.fail ())
-          {
-            cerr << "Missing " << version_file_name << endl;
-            exit(1);
-          }
-        else
-          {
-            char c;
-            s.kernel_release = "";
-            while (version_file.get(c) && c != '\n')
-              s.kernel_release.push_back(c);
-          }
-      }
-    else
-      {
-        s.kernel_release = string (s_kr);
-        s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
-      }
+    setup_kernel_release(s, s_kr);
   }
 
 
   while (true)
     {
       int long_opt;
+      char * num_endptr;
 #define LONG_OPT_KELF 1
 #define LONG_OPT_KMAP 2
 #define LONG_OPT_IGNORE_VMLINUX 3
@@ -506,8 +523,8 @@ main (int argc, char * const argv [])
               cerr << "Listing (-l) mode implies pass 2." << endl;
               usage (s, 1);
             }
-          s.last_pass = atoi (optarg);
-          if (s.last_pass < 1 || s.last_pass > 5)
+          s.last_pass = (int)strtoul(optarg, &num_endptr, 10);
+          if (*num_endptr != '\0' || s.last_pass < 1 || s.last_pass > 5)
             {
               cerr << "Invalid pass number (should be 1-5)." << endl;
               usage (s, 1);
@@ -588,31 +605,7 @@ main (int argc, char * const argv [])
           break;
 
         case 'r':
-          if (optarg[0] == '/') // fully specified path
-            {
-              s.kernel_build_tree = optarg;
-              string version_file_name = s.kernel_build_tree + "/include/config/kernel.release";
-              // The file include/config/kernel.release within the
-              // build tree is used to pull out the version information
-              ifstream version_file (version_file_name.c_str());
-              if (version_file.fail ())
-                {
-                  cerr << "Missing " << version_file_name << endl;
-                  usage (s, 1);
-                }
-              else 
-                {
-                  char c;
-                  s.kernel_release = "";
-                  while (version_file.get(c) && c != '\n')
-                    s.kernel_release.push_back(c);
-                }
-            }
-          else
-            {
-              s.kernel_release = string (optarg);
-              s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
-            }
+          setup_kernel_release(s, optarg);
           break;
 
         case 'k':
@@ -637,8 +630,8 @@ main (int argc, char * const argv [])
 	  break;
 
         case 's':
-          s.buffer_size = atoi (optarg);
-          if (s.buffer_size < 1 || s.buffer_size > 4095)
+          s.buffer_size = (int) strtoul (optarg, &num_endptr, 10);
+          if (*num_endptr != '\0' || s.buffer_size < 1 || s.buffer_size > 4095)
             {
               cerr << "Invalid buffer size (should be 1-4095)." << endl;
 	      usage (s, 1);
@@ -650,7 +643,12 @@ main (int argc, char * const argv [])
 	  break;
 
 	case 'x':
-	  s.target_pid = atoi(optarg);
+	  s.target_pid = (int) strtoul(optarg, &num_endptr, 10);
+	  if (*num_endptr != '\0')
+	    {
+	      cerr << "Invalid target process ID number." << endl;
+	      usage (s, 1);
+	    }
 	  break;
 
 	case 'D':
@@ -670,6 +668,7 @@ main (int argc, char * const argv [])
           break;
 
         case 'L':
+          s.listing_mode_vars = true;
           s.unoptimized = true; // This causes retention of variables for listing_mode
 
         case 'l':
@@ -822,6 +821,8 @@ main (int argc, char * const argv [])
   
   // PASS 0: setting up
   s.verbose = s.perpass_verbose[0];
+  STAP_PROBE1(stap, pass0__start, &s);
+
 
   // For PR1477, we used to override $PATH and $LC_ALL and other stuff
   // here.  We seem to use complete pathnames in
@@ -873,12 +874,15 @@ main (int argc, char * const argv [])
   // and reasonably timely exit.
   setup_signals(&handle_interrupt);
 
+  STAP_PROBE1(stap, pass0__end, &s);
+
   struct tms tms_before;
   times (& tms_before);
   struct timeval tv_before;
   gettimeofday (&tv_before, NULL);
 
   // PASS 1a: PARSING USER SCRIPT
+  STAP_PROBE1(stap, pass1a__start, &s);
 
   struct stat user_file_stat;
   int user_file_stat_rc = -1;
@@ -930,6 +934,8 @@ main (int argc, char * const argv [])
   version_suffixes.push_back ("");
 
   // PASS 1b: PARSING LIBRARY SCRIPTS
+  STAP_PROBE1(stap, pass1b__start, &s);
+
   for (unsigned i=0; i<s.include_path.size(); i++)
     {
       // now iterate upon it
@@ -1017,6 +1023,8 @@ main (int argc, char * const argv [])
          << "Try again with another '--vp 1' option."
          << endl;
 
+  STAP_PROBE1(stap, pass1__end, &s);
+
   if (rc || s.last_pass == 1 || pending_interrupts) goto cleanup;
 
   times (& tms_before);
@@ -1024,6 +1032,7 @@ main (int argc, char * const argv [])
 
   // PASS 2: ELABORATION
   s.verbose = s.perpass_verbose[1];
+  STAP_PROBE1(stap, pass2__start, &s);
   rc = semantic_pass (s);
 
   if (s.listing_mode || (rc == 0 && s.last_pass == 2))
@@ -1075,12 +1084,18 @@ main (int argc, char * const argv [])
 	}
     }
 
+  /* Print out list of missing files */
+  missing_rpm_list_print(s);
+
+  STAP_PROBE1(stap, pass2__end, &s);
+
   if (rc || s.listing_mode || s.last_pass == 2 || pending_interrupts) goto cleanup;
 
   // PASS 3: TRANSLATION
   s.verbose = s.perpass_verbose[2];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
+  STAP_PROBE1(stap, pass3__start, &s);
 
   rc = translate_pass (s);
 
@@ -1104,12 +1119,15 @@ main (int argc, char * const argv [])
          << "Try again with another '--vp 001' option."
          << endl;
 
+  STAP_PROBE1(stap, pass3__end, &s);
+
   if (rc || s.last_pass == 3 || pending_interrupts) goto cleanup;
 
   // PASS 4: COMPILATION
   s.verbose = s.perpass_verbose[3];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
+  STAP_PROBE1(stap, pass4__start, &s);
   rc = compile_pass (s);
 
   if (rc == 0 && s.last_pass == 4)
@@ -1154,9 +1172,25 @@ main (int argc, char * const argv [])
 	  if (copy_file(module_src_path.c_str(), module_dest_path.c_str()) != 0)
 	    cerr << "Copy failed (\"" << module_src_path << "\" to \""
 		 << module_dest_path << "\"): " << strerror(errno) << endl;
+
+#if HAVE_NSS
+	  // Save the signature as well.
+	  assert (! s.cert_db_path.empty());
+	  module_src_path += ".sgn";
+	  module_dest_path += ".sgn";
+
+	  if (s.verbose > 1)
+	    clog << "Copying " << module_src_path << " to "
+		 << module_dest_path << endl;
+	  if (copy_file(module_src_path.c_str(), module_dest_path.c_str()) != 0)
+	    cerr << "Copy failed (\"" << module_src_path << "\" to \""
+		 << module_dest_path << "\"): " << strerror(errno) << endl;
+#endif
 	}
     }
 
+  STAP_PROBE1(stap, pass4__end, &s);
+
   if (rc || s.last_pass == 4 || pending_interrupts) goto cleanup;
 
 
@@ -1168,6 +1202,7 @@ pass_5:
   // NB: this message is a judgement call.  The other passes don't emit
   // a "hello, I'm starting" message, but then the others aren't interactive
   // and don't take an indefinite amount of time.
+  STAP_PROBE1(stap, pass5__start, &s);
   if (s.verbose) clog << "Pass 5: starting run." << endl;
   rc = run_pass (s);
   times (& tms_after);
@@ -1183,9 +1218,13 @@ pass_5:
 
   // if (rc) goto cleanup;
 
+  STAP_PROBE1(stap, pass5__end, &s);
+
   // PASS 6: cleaning up
  cleanup:
 
+  STAP_PROBE1(stap, pass6__start, &s);
+
   // update the database information
   if (!rc && s.tapset_compile_coverage && !pending_interrupts) {
 #ifdef HAVE_LIBSQLITE3
@@ -1217,6 +1256,8 @@ pass_5:
         }
     }
 
+  STAP_PROBE1(stap, pass6__end, &s);
+
   return (rc||pending_interrupts) ? EXIT_FAILURE : EXIT_SUCCESS;
 }
 
