@@ -14,7 +14,9 @@
 
 #include "string.h"
 #include "vsprintf.c"
+#include "print.h"
 #include "transport/transport.c"
+#include "vsprintf.c"
 
 /** @file print.c
  * Printing Functions.
@@ -86,14 +88,8 @@ static void _stp_print_cleanup (void)
 #endif
 
 #if !defined(RELAY_GUEST)
-/* The relayfs API changed between 2.6.15 and 2.6.16. */
-/* Use the appropriate print flush function. */
 
-#ifdef STP_OLD_TRANSPORT
-#include "print_old.c"
-#else
-#include "print_new.c"
-#endif
+#include "print_flush.c"
 #if defined(RELAY_HOST)
 EXPORT_SYMBOL_GPL(EXPORT_FN(stp_print_flush));
 #endif
@@ -173,34 +169,10 @@ static void _stp_print_binary (int num, ...)
  */
 static void _stp_printf (const char *fmt, ...)
 {
-	int num;
 	va_list args;
-	_stp_pbuf *pb = per_cpu_ptr(Stp_pbuf, smp_processor_id());
-	char *buf = pb->buf + pb->len;
-	int size = STP_BUFFER_SIZE - pb->len;
-
 	va_start(args, fmt);
-	num = _stp_vsnprintf(buf, size, fmt, args);
+	_stp_vsnprintf(NULL, 0, fmt, args);
 	va_end(args);
-	if (unlikely(num >= size)) { 
-		/* overflowed the buffer */
-		if (pb->len == 0) {
-			/* A single print request exceeded the buffer size. */
-			/* Should not be possible with Systemtap-generated code. */
-			pb->len = STP_BUFFER_SIZE;
-			_stp_print_flush();
-			num = 0;
-		} else {
-			/* Need more space. Flush the previous contents */
-			_stp_print_flush();
-			
-			/* try again */
-			va_start(args, fmt);
-			num = _stp_vsnprintf(pb->buf, STP_BUFFER_SIZE, fmt, args);
-			va_end(args);
-		}
-	}
-	pb->len += num;
 }
 
 /** Write a string into the print buffer.
