@@ -34,17 +34,25 @@
 #if !defined (CONFIG_DEBUG_FS)  && !defined (CONFIG_DEBUG_FS_MODULE)
 #error "DebugFS is required and was not found in the kernel."
 #endif
+#ifdef CONFIG_RING_BUFFER
+#define STP_TRANSPORT_VERSION 3
+#else
+#define STP_TRANSPORT_VERSION 2
+#endif
 #else
 /* older kernels have no debugfs and older version of relayfs. */
-#define STP_OLD_TRANSPORT
+#define STP_TRANSPORT_VERSION 1
 #endif
 
-#ifndef for_each_cpu
-#define for_each_cpu(cpu)  for_each_cpu_mask((cpu), cpu_possible_map)
+#ifndef stp_for_each_cpu
+#define stp_for_each_cpu(cpu)  for_each_cpu_mask((cpu), cpu_possible_map)
 #endif
 
 static void _stp_dbug (const char *func, int line, const char *fmt, ...);
-void _stp_error (const char *fmt, ...);
+static void _stp_error (const char *fmt, ...);
+static void _stp_warn (const char *fmt, ...);
+
+static void _stp_exit(void);
 
 #include "debug.h"
 
@@ -66,6 +74,13 @@ static struct
 #define MAXTRACE 20
 #endif
 
+/* dwarf unwinder only tested so far on i386 and x86_64. */
+#if (defined(__i386__) || defined(__x86_64__))
+#ifndef STP_USE_DWARF_UNWINDER
+#define STP_USE_DWARF_UNWINDER
+#endif
+#endif
+
 #ifdef CONFIG_FRAME_POINTER
 /* Just because frame pointers are available does not mean we can trust them. */
 #ifndef STP_USE_DWARF_UNWINDER
@@ -73,19 +88,18 @@ static struct
 #endif
 #endif
 
-/* dwarf unwinder only tested so far on i386 and x86_64,
-   but globally disabled for now */
-#if 0
-// !defined(STP_USE_FRAME_BUFFER) && (defined(__i386__) || defined(__x86_64__))
-#define STP_USE_DWARF_UNWINDER
-#endif
-
 #include "alloc.c"
 #include "print.c"
 #include "string.c"
 #include "io.c"
 #include "arith.c"
 #include "copy.c"
+#include "regs.c"
+#include "regs-ia64.c"
+#include "time.c"
+
+#include "task_finder.c"
+
 #include "sym.c"
 #ifdef STP_PERFMON
 #include "perf.c"
@@ -93,7 +107,7 @@ static struct
 #include "addr-map.c"
 
 /* Support functions for int64_t module parameters. */
-int param_set_int64_t(const char *val, struct kernel_param *kp)
+static int param_set_int64_t(const char *val, struct kernel_param *kp)
 {
   char *endp;
   long long ll;
@@ -114,7 +128,7 @@ int param_set_int64_t(const char *val, struct kernel_param *kp)
   return 0;
 }
 
-int param_get_int64_t(char *buffer, struct kernel_param *kp)
+static int param_get_int64_t(char *buffer, struct kernel_param *kp)
 {
   return sprintf(buffer, "%lli", (long long)*((int64_t *)kp->arg));
 }
@@ -129,7 +143,7 @@ int init_module (void)
   return _stp_transport_init();
 }
 
-int probe_start(void);
+static int probe_start(void);
 
 void cleanup_module(void)
 {
