@@ -12,6 +12,7 @@
 #ifndef _STP_SYM_C_
 #define _STP_SYM_C_
 
+#include "sym.h"
 #include "string.c"
 #include "task_finder_vma.c"
 
@@ -271,34 +272,29 @@ static int _stp_module_check(void)
                        dwfl_module_build_id was not intended to return the end address. */
 		    notes_addr -= m->build_id_len;
 
-		    if (notes_addr > base_addr) {
-		      for (j = 0; j < m->build_id_len; j++)	 
-                        {
-                          unsigned char theory, practice;
-                          theory = m->build_id_bits [j];
-                          practice = ((unsigned char*) notes_addr) [j];
-                          /* XXX: consider using kread() instead of above. */
-                          if (theory != practice)
-                            {
-                              #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
-                              _stp_error ("%s: inconsistent %s build-id byte #%d "
-                                          "(0x%x [actual] vs. 0x%x [debuginfo])\n",
-                                          THIS_MODULE->name, m->name, j,
-                                          practice, theory);
-                              return 1;
-                              #else
-                              /* This branch is a surrogate for
-                                 kernels affected by Fedora bug
-                                 #465873. */
-                              printk(KERN_WARNING
-                                     "%s: inconsistent %s build-id byte #%d "
-                                     "(0x%x [actual] vs. 0x%x [debuginfo])\n",
-                                     THIS_MODULE->name, m->name, j,
-                                     practice, theory);
-                              break; /* Note just the first mismatch. */
-                              #endif
-                            }
-			} 
+		    if (notes_addr <= base_addr)  /* shouldn't happen */
+			 continue;
+		    if (memcmp(m->build_id_bits, (unsigned char*) notes_addr, m->build_id_len)) {
+	                 const char *basename;
+
+			 basename = strrchr(m->path, '/');
+			 if (basename)
+			     basename++;
+			 else
+			     basename = m->path;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
+                         _stp_error ("Build-id mismatch: \"%s\" %.*M"
+				     " vs. \"%s\" %.*M\n",
+				     m->name, m->build_id_len, notes_addr,
+				     basename, m->build_id_len, m->build_id_bits);
+                         return 1;
+#else
+                         /* This branch is a surrogate for kernels
+			  * affected by Fedora bug #465873. */
+                         printk(KERN_WARNING
+				 "Build-id mismatch: \"%s\" vs. \"%s\"\n",
+				 m->name, basename);
+#endif
 		    }
 		} /* end checking */
 	} /* end loop */
@@ -313,9 +309,10 @@ static int _stp_module_check(void)
 
 static void _stp_symbol_print(unsigned long address)
 {
-	const char *modname;
-	const char *name;
-	unsigned long offset, size;
+	const char *modname = 0;
+	const char *name = 0;
+	unsigned long offset = 0;
+        unsigned long size = 0;
 
 	name = _stp_kallsyms_lookup(address, &size, &offset, &modname, NULL, NULL);
 
@@ -338,9 +335,10 @@ static void _stp_symbol_print(unsigned long address)
 
 static void _stp_usymbol_print(unsigned long address, struct task_struct *task)
 {
-	const char *modname;
-	const char *name;
-	unsigned long offset, size;
+	const char *modname = 0;
+	const char *name = 0;
+	unsigned long offset = 0;
+        unsigned long size = 0;
 
 	name = _stp_kallsyms_lookup(address, &size, &offset, &modname, NULL,
                                     task);
