@@ -89,6 +89,7 @@ struct systemtap_session
   std::string architecture;
   std::string runtime_path;
   std::string data_path;
+  std::string cert_db_path;
   std::string module_name;
   std::string stapconf_name;
   std::string output_file;
@@ -102,6 +103,7 @@ struct systemtap_session
   bool keep_tmpdir;
   bool guru_mode;
   bool listing_mode;
+  bool listing_mode_vars;
   bool bulk_mode;
   bool unoptimized;
   bool merge;
@@ -123,7 +125,6 @@ struct systemtap_session
   std::string cache_path;
   std::string hash_path;
   std::string stapconf_path;
-  std::string tracequery_path;
 
   // dwarfless operation
   bool consult_symtab;
@@ -210,6 +211,8 @@ struct systemtap_session
   std::set<std::string> seen_warnings;
   unsigned num_errors () { return seen_errors.size(); }
 
+  std::set<std::string> rpms_to_install;
+
   // void print_error (const parse_error& e);
   const token* last_token;
   void print_token (std::ostream& o, const token* tok);
