@@ -1,5 +1,5 @@
 // Copyright (C) Andrew Tridgell 2002 (original file)
-// Copyright (C) 2006 Red Hat Inc. (systemtap changes)
+// Copyright (C) 2006, 2009 Red Hat Inc. (systemtap changes)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -16,6 +16,7 @@
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 #include "util.h"
+#include "sys/sdt.h"
 #include <stdexcept>
 #include <cerrno>
 
@@ -135,6 +136,25 @@ create_dir(const char *dir)
   return 0;
 }
 
+// Remove a file or directory
+int
+remove_file_or_dir (const char *name)
+{
+  int rc;
+  struct stat st;
+
+  if ((rc = stat(name, &st)) != 0)
+    {
+      if (errno == ENOENT)
+	return 0;
+      return 1;
+    }
+
+  if (remove (name) != 0)
+    return 1;
+  cerr << "remove returned 0" << endl;
+  return 0;
+}
 
 void
 tokenize(const string& str, vector<string>& tokens,
@@ -284,12 +304,14 @@ static pid_t spawned_pid = 0;
 int
 stap_system(const char *command)
 {
+  STAP_PROBE1(stap, stap_system__start, command);
   const char * argv[] = { "sh", "-c", command, NULL };
   int ret, status;
 
   spawned_pid = 0;
   ret = posix_spawn(&spawned_pid, "/bin/sh", NULL, NULL,
                     const_cast<char **>(argv), environ);
+  STAP_PROBE2(stap, stap_system__spawn, ret, spawned_pid);
   if (ret == 0)
     {
       if (waitpid(spawned_pid, &status, 0) == spawned_pid)
@@ -297,6 +319,7 @@ stap_system(const char *command)
       else
         ret = errno;
     }
+  STAP_PROBE1(stap, stap_system__complete, ret);
   spawned_pid = 0;
   return ret;
 }
