@@ -8,6 +8,7 @@
 const char *get_home_directory(void);
 int copy_file(const char *src, const char *dest);
 int create_dir(const char *dir);
+int remove_file_or_dir(const char *dir);
 void tokenize(const std::string& str, std::vector<std::string>& tokens,
 	      const std::string& delimiters);
 std::string find_executable(const std::string& name);
@@ -69,7 +70,7 @@ lex_cast_qstring(IN const & in)
   out2 += '"';
   for (unsigned i=0; i<out.length(); i++)
     {
-      char c = out[i];
+      unsigned char c = out[i];
       if (! isprint(c))
         {
           out2 += '\\';
